library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

-- transcodeur specifique pour reveil : binaire 4 bits vers information logique sur 7 bits
--  (indiquant si le segment A, B... G est allume ou non)
--  0 à 9 : dessin des chiffres
--	 10 : dessin du A de alarme
--	 11 : dessin du L de alarme
--	 12 à 14 : libre 
--  15 : tous les segments éteints pour un effet de clignotement éventuel
--  entrée hl : pour gérer les led en anode commune ou cathode commune


entity cpt_horloge is port (
	clk,load,clr,en : in std_logic;	
	max : in std_logic_vector(3 downto 0);		-- valeur max du compteur
	din : in std_logic_vector(3 downto 0);		-- valeur de prechargement du compteur
	enout : out std_logic; 
	s : out std_logic_vector(3 downto 0) 	
	);
end entity;

architecture behavior of cpt_horloge is
signal q : std_logic_vector(3 downto 0);
begin
process(clr,load,clk,din) begin
	if clr='1' then
		q <= (others => '0');
	elsif (rising_edge(clk)) then
		if load='1' then
				q <= din;
		elsif en='1' then
			if q < max then
				q <= q+1;
			else
				q <= (others=> '0' );
			end if;
		end if;
	end if;
end process;

process(en,q) begin
	if (en='1' and q=max) then
		enout <= '1';
	else
		enout <= '0';
	end if;
end process;

s <= q;
end behavior;