library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

entity top is port (
clk : in std_logic;
hl,init,sw1 : in std_logic;
led : out std_logic;
seg_hd,seg_hu,seg_md,seg_mu,seg_sd,seg_su,seg_al : out std_logic_vector(6 downto 0)
);
end entity;

architecture behavior of top is

component transco_reveil is port (
	e : in std_logic_vector(3 downto 0);	-- valeur en entree a afficher
	hl : in std_logic;		-- gestion anode commune ou cathode commune 
	s : out std_logic_vector(6 downto 0) 	--	gestion des segments
	);
end component;

component cpt_horloge is port (
	clk,load,clr,en : in std_logic;	
	max : in std_logic_vector(3 downto 0);		-- valeur max du compteur
	din : in std_logic_vector(3 downto 0);		-- valeur de prechargement du compteur
	enout : out std_logic; 
	s : out std_logic_vector(3 downto 0) 	
	);
end component;

component DDS is
		port(
			clk: in std_logic;
			dn: in std_logic_vector(31 downto 0);
			clk_out : out std_logic
				);
		end component;

signal clk1hz,clk2hz,clk4hz,clk10hz,clk200hz : std_logic;
signal hsu,hsd,hmu,hmd,hhu,hhd,alarme : std_logic_vector(3 downto 0);
signal endday : std_logic;
signal ensd,enmu,enmd,enhu,enhd : std_logic;
signal match : std_logic;

type state is (off,arme,sonnerie); 
signal statemode: state;

begin

-- generateurs de frequence
quartz_1hz : dds port map (clk=>clk, clk_out=> clk1hz, dn =>x"00000056");
quartz_2hz : dds port map (clk=>clk, clk_out=> clk2hz, dn =>x"000000AC");
quartz_4hz : dds port map (clk=>clk, clk_out=> clk4hz, dn =>x"00000157");
quartz_10hz : dds port map (clk=>clk, clk_out=> clk10hz, dn =>x"0000035B");
quartz_200hz : dds port map (clk=>clk, clk_out=> clk200hz, dn =>x"0000431C");
-- fsm pour les modes de fonctionnement ( horloge, reglage heure, reglage alarme)
		fsm_modes : process(clk200hz) begin
			if rising_edge(clk200hz) then
				if init='0' then
					statemode <= off;
				else
					case statemode is
						when off =>	if (sw1='1') then 
											statemode<=arme;
										else
											statemode <= off;
										end if;
						when arme => if (sw1='1' and match='1') then 
											statemode<=sonnerie;
										elsif (sw1='0') then
											statemode <= off;
										else
											statemode <= arme;
										end if;
						when sonnerie => 	if (sw1='0') then 
											statemode<= off;
										else
											statemode <= sonnerie; 
										end if;
						when others => statemode	<= off; -- par defaut pas de sonnerie
					end case;
				end if;
			end if;
		end process;

-- actions de la fsm

	-- sonnerie : indicateur visuel ici
	process(statemode) begin
		if statemode=sonnerie then
			led<='1';
		else
			led<='0';
		end if;
	end process;
	
	-- réveil armé pour sonner 
	process(statemode) begin
		if statemode=off then
			alarme<="1111"; -- A eteint
		else
			alarme<="1010"; -- A allumé
		end if;
	end process;

-- detection heure de sonnerie
	process(hsu,hsd,hmu,hmd,hhu,hhd) begin
	 if (hsu=9 and hsd=5 and hmu=4 and hmd=0 and hhu=0 and hhd=0) then
		match <= '1';
	 else
		match <= '0';
	 end if;
	end process;

-- transcodeurs pour affichage
aff_su: transco_reveil port map (e=>hsu,hl=>hl,s=>seg_su);
aff_sd: transco_reveil port map (e=>hsd,hl=>hl,s=>seg_sd);
aff_mu: transco_reveil port map (e=>hmu,hl=>hl,s=>seg_mu);
aff_md: transco_reveil port map (e=>hmd,hl=>hl,s=>seg_md);
aff_hu: transco_reveil port map (e=>hhu,hl=>hl,s=>seg_hu);
aff_hd: transco_reveil port map (e=>hhd,hl=>hl,s=>seg_hd);

aff_alarme : transco_reveil port map (e=>alarme,hl=>hl,s=>seg_al);

-- compteurs horloge
cpt_su : cpt_horloge port map(clk=> clk10hz, load=>not init, clr=>endday, en=>'1', max=> "1001", din=> "0000", enout=> ensd, s=> hsu);
cpt_sd : cpt_horloge port map(clk=> clk10hz, load=>not init, clr=>endday, en=>ensd, max=> "0101", din=> "0000", enout=> enmu, s=> hsd);
cpt_mu : cpt_horloge port map(clk=> clk10hz, load=>not init, clr=>endday, en=>enmu, max=> "1001", din=> "0000", enout=> enmd, s=> hmu);
cpt_md : cpt_horloge port map(clk=> clk10hz, load=>not init, clr=>endday, en=>enmd, max=> "0101", din=> "0000", enout=> enhu, s=> hmd);
cpt_hu : cpt_horloge port map(clk=> clk10hz, load=>not init, clr=>endday, en=>enhu, max=> "1001", din=> "0000", enout=> enhd, s=> hhu);
cpt_hd : cpt_horloge port map(clk=> clk10hz, load=>not init, clr=>endday, en=>enhd, max=> "0010", din=> "0000", s=> hhd);

-- detection de 23:59:59
process(hsu,hsd,hmu,hmd,hhu,hhd) begin
 if (hsu=9 and hsd=5 and hmu=9 and hmd=5 and hhu=3 and hhd=2) then
	endday <= '1';
 else
	endday <= '0';
 end if;
end process;
	
end behavior;

